@@ -1,4 +1,4 @@
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TemplateHaskell, StandaloneDeriving #-}
 
 {-| Implementation of opcodes parameters.
 
@@ -68,6 +68,7 @@ module Ganeti.OpParams
   , pShutdownInstance
   , pForce
   , pIgnoreOfflineNodes
+  , pIgnoreHVVersions
   , pNodeName
   , pNodeUuid
   , pNodeNames
@@ -115,6 +116,7 @@ module Ganeti.OpParams
   , pInstDisks
   , pDiskTemplate
   , pOptDiskTemplate
+  , pExtParams
   , pFileDriver
   , pFileStorageDir
   , pClusterFileStorageDir
@@ -188,6 +190,7 @@ module Ganeti.OpParams
   , pRemoteNode
   , pRemoteNodeUuid
   , pEvacMode
+  , pIgnoreSoftErrors
   , pInstCreateMode
   , pNoInstall
   , pInstOs
@@ -203,6 +206,8 @@ module Ganeti.OpParams
   , pSrcNodeUuid
   , pSrcPath
   , pStartInstance
+  , pForthcoming
+  , pCommit
   , pInstTags
   , pMultiAllocInstances
   , pTempOsParams
@@ -291,6 +296,13 @@ module Ganeti.OpParams
   , pEnabledDiskTemplates
   , pEnabledUserShutdown
   , pAdminStateSource
+  , pEnabledDataCollectors
+  , pDataCollectorInterval
+  , pNodeSslCerts
+  , pSshKeys
+  , pNodeSetup
+  , pVerifyClutter
+  , pLongSleep
   ) where
 
 import Control.Monad (liftM, mplus)
@@ -382,6 +394,8 @@ $(buildObject "INicParams" "inic"
   , optionalField $ simpleField C.inicNetwork [t| NonEmptyString |]
   ])
 
+deriving instance Ord INicParams
+
 -- | Disk modification definition.
 $(buildObject "IDiskParams" "idisk"
   [ specialNumericalField 'parseUnitAssumeBinary . optionalField
@@ -393,9 +407,12 @@ $(buildObject "IDiskParams" "idisk"
   , optionalField $ simpleField C.idiskName   [t| NonEmptyString |]
   , optionalField $ simpleField C.idiskProvider [t| NonEmptyString |]
   , optionalField $ simpleField C.idiskSpindles [t| Int          |]
+  , optionalField $ simpleField C.idiskAccess   [t| NonEmptyString |]
   , andRestArguments "opaque"
   ])
 
+deriving instance Ord IDiskParams
+
 -- | Disk changes type for OpInstanceRecreateDisks. This is a bit
 -- strange, because the type in Python is something like Either
 -- [DiskIndex] [DiskChanges], but we can't represent the type of an
@@ -405,7 +422,7 @@ data RecreateDisksInfo
   = RecreateDisksAll
   | RecreateDisksIndices (NonEmpty DiskIndex)
   | RecreateDisksParams (NonEmpty (DiskIndex, IDiskParams))
-    deriving (Eq, Show)
+    deriving (Eq, Show, Ord)
 
 readRecreateDisks :: JSValue -> Text.JSON.Result RecreateDisksInfo
 readRecreateDisks (JSArray []) = return RecreateDisksAll
@@ -427,7 +444,7 @@ instance JSON RecreateDisksInfo where
 -- | Simple type for old-style ddm changes.
 data DdmOldChanges = DdmOldIndex (NonNegative Int)
                    | DdmOldMod DdmSimple
-                     deriving (Eq, Show)
+                     deriving (Eq, Show, Ord)
 
 readDdmOldChanges :: JSValue -> Text.JSON.Result DdmOldChanges
 readDdmOldChanges v =
@@ -449,7 +466,7 @@ data SetParamsMods a
   | SetParamsDeprecated (NonEmpty (DdmOldChanges, a))
   | SetParamsNew (NonEmpty (DdmFull, Int, a))
   | SetParamsNewName (NonEmpty (DdmFull, String, a))
-    deriving (Eq, Show)
+    deriving (Eq, Show, Ord)
 
 -- | Custom deserialiser for 'SetParamsMods'.
 readSetParams :: (JSON a) => JSValue -> Text.JSON.Result (SetParamsMods a)
@@ -472,7 +489,7 @@ instance (JSON a) => JSON (SetParamsMods a) where
 -- tests). But the proper type could be parsed if we wanted.
 data ExportTarget = ExportTargetLocal NonEmptyString
                   | ExportTargetRemote [JSValue]
-                    deriving (Eq, Show)
+                    deriving (Eq, Show, Ord)
 
 -- | Custom reader for 'ExportTarget'.
 readExportTarget :: JSValue -> Text.JSON.Result ExportTarget
@@ -860,14 +877,14 @@ pNodeUuids =
 
 pOobCommand :: Field
 pOobCommand =
-  withDoc "OOB command to run" $
-  simpleField "command" [t| OobCommand |]
+  withDoc "OOB command to run" .
+  renameField "OobCommand" $ simpleField "command" [t| OobCommand |]
 
 pOobTimeout :: Field
 pOobTimeout =
   withDoc "Timeout before the OOB helper will be terminated" .
-  defaultField [| C.oobTimeout |] $
-  simpleField "timeout" [t| Int |]
+  defaultField [| C.oobTimeout |] .
+  renameField "OobTimeout" $ simpleField "timeout" [t| Int |]
 
 pIgnoreStatus :: Field
 pIgnoreStatus =
@@ -983,6 +1000,11 @@ pIgnoreConsistency =
   withDoc "Whether to ignore disk consistency" $
   defaultFalse "ignore_consistency"
 
+pIgnoreHVVersions :: Field
+pIgnoreHVVersions =
+  withDoc "Whether to ignore incompatible Hypervisor versions" $
+  defaultFalse "ignore_hvversions"
+
 pMasterCandidate :: Field
 pMasterCandidate =
   withDoc "Whether the node should become a master candidate" .
@@ -1067,6 +1089,12 @@ pEvacMode =
   withDoc "Node evacuation mode" .
   renameField "EvacMode" $ simpleField "mode" [t| EvacMode |]
 
+pIgnoreSoftErrors :: Field
+pIgnoreSoftErrors =
+  withDoc "Ignore soft htools errors" .
+  optionalField $
+  booleanField "ignore_soft_errors"
+
 pInstanceName :: Field
 pInstanceName =
   withDoc "A required instance name (for single-instance LUs)" $
@@ -1116,6 +1144,13 @@ pDiskTemplate =
   withDoc "Disk template" $
   simpleField "disk_template" [t| DiskTemplate |]
 
+pExtParams :: Field
+pExtParams =
+  withDoc "List of ExtStorage parameters" .
+  renameField "InstExtParams" .
+  defaultField [| toJSObject [] |] $
+  simpleField "ext_params" [t| JSObject JSValue |]
+
 pFileDriver :: Field
 pFileDriver =
   withDoc "Driver for file-backed disks" .
@@ -1256,6 +1291,16 @@ pStartInstance =
   withDoc "Whether to start instance after creation" $
   defaultTrue "start"
 
+pForthcoming :: Field
+pForthcoming =
+  withDoc "Whether to only reserve resources" $
+  defaultFalse "forthcoming"
+
+pCommit :: Field
+pCommit =
+  withDoc "Commit the already reserved instance" $
+  defaultFalse "commit"
+
 -- FIXME: unify/simplify with pTags, once that migrates to NonEmpty String"
 pInstTags :: Field
 pInstTags =
@@ -1588,7 +1633,8 @@ pTagsName =
 
 pTagsList :: Field
 pTagsList =
-  withDoc "List of tag names" $
+  withDoc "List of tag names" .
+  renameField "TagsList" $
   simpleField "tags" [t| [String] |]
 
 -- FIXME: this should be compiled at load time?
@@ -1830,3 +1876,45 @@ pNetworkVlan :: Field
 pNetworkVlan =
   withDoc "Network vlan when connecting to a group" .
   defaultField [| "" |] $ stringField "network_vlan"
+
+pEnabledDataCollectors :: Field
+pEnabledDataCollectors =
+  withDoc "Set the active data collectors" .
+  optionalField $
+  simpleField C.dataCollectorsEnabledName [t| GenericContainer String Bool |]
+
+pDataCollectorInterval :: Field
+pDataCollectorInterval =
+  withDoc "Sets the interval in that data collectors are run" .
+  optionalField $
+  simpleField C.dataCollectorsIntervalName [t| GenericContainer String Int |]
+
+pNodeSslCerts :: Field
+pNodeSslCerts =
+  withDoc "Whether to renew node SSL certificates" .
+  defaultField [| False |] $
+  simpleField "node_certificates" [t| Bool |]
+
+pSshKeys :: Field
+pSshKeys =
+  withDoc "Whether to renew SSH keys" .
+  defaultField [| False |] $
+  simpleField "ssh_keys" [t| Bool |]
+
+pNodeSetup :: Field
+pNodeSetup =
+  withDoc "Whether to perform a SSH setup on the new node" .
+  defaultField [| False |] $
+  simpleField "node_setup" [t| Bool |]
+
+pVerifyClutter :: Field
+pVerifyClutter =
+  withDoc "Whether to check for clutter in the 'authorized_keys' file." .
+  defaultField [| False |] $
+  simpleField "verify_clutter" [t| Bool |]
+
+pLongSleep :: Field
+pLongSleep =
+  withDoc "Whether to allow long instance shutdowns during exports" .
+  defaultField [| False |] $
+  simpleField "long_sleep" [t| Bool |]
\ No newline at end of file
