@@ -38,6 +38,7 @@ module Test.Ganeti.Query.Instance
   ( testQuery_Instance
   ) where
 
+import qualified Data.ByteString.UTF8 as UTF8
 import qualified Data.Map as Map
 import qualified Data.Set as Set
 import System.Time (ClockTime(..))
@@ -63,7 +64,7 @@ createInstance name pnodeUuid adminState adminStateSource =
     (PartialBeParams Nothing Nothing Nothing Nothing Nothing Nothing)
     (GenericContainer Map.empty) (GenericContainer Map.empty)
     adminState adminStateSource [] [] False Nothing epochTime epochTime
-    "" 0 Set.empty
+    (UTF8.fromString "") 0 Set.empty
   where epochTime = TOD 0 0
 
 -- | A fake InstanceInfo to be used to check values.
