@@ -1,4 +1,4 @@
-{-# LANGUAGE TemplateHaskell, CPP #-}
+{-# LANGUAGE TemplateHaskell, CPP, ScopedTypeVariables #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 {-| Unittests for ganeti-htools.
@@ -40,6 +40,7 @@ module Test.Ganeti.Utils (testUtils) where
 import Test.QuickCheck hiding (Result)
 import Test.HUnit
 
+import Control.Applicative ((<$>), (<*>))
 import Data.Char (isSpace)
 import qualified Data.Either as Either
 import Data.List
@@ -47,7 +48,7 @@ import Data.Maybe (listToMaybe)
 import qualified Data.Set as S
 import System.Time
 import qualified Text.JSON as J
-#ifndef NO_REGEX_PCRE
+#ifdef VERSION_regex_pcre
 import Text.Regex.PCRE
 #endif
 
@@ -59,6 +60,9 @@ import qualified Ganeti.Constants as C
 import qualified Ganeti.JSON as JSON
 import Ganeti.Utils
 
+{-# ANN module "HLint: ignore Use camelCase" #-}
+
+
 -- | Helper to generate a small string that doesn't contain commas.
 genNonCommaString :: Gen String
 genNonCommaString = do
@@ -260,12 +264,18 @@ prop_rStripSpace (NonEmpty str) =
               rStripSpace "" ==? ""
           ]
 
-#ifndef NO_REGEX_PCRE
-{-# ANN case_new_uuid "HLint: ignore Use camelCase" #-}
-
 -- | Tests that the newUUID function produces valid UUIDs.
 case_new_uuid :: Assertion
 case_new_uuid = do
+  uuid <- newUUID
+  assertBool "newUUID" $ isUUID uuid
+
+#ifdef VERSION_regex_pcre
+{-# ANN case_new_uuid_regex "HLint: ignore Use camelCase" #-}
+
+-- | Tests that the newUUID function produces valid UUIDs.
+case_new_uuid_regex :: Assertion
+case_new_uuid_regex = do
   uuid <- newUUID
   assertBool "newUUID" $ uuid =~ C.uuidRegex
 #endif
@@ -274,6 +284,17 @@ prop_clockTimeToString :: Integer -> Integer -> Property
 prop_clockTimeToString ts pico =
   clockTimeToString (TOD ts pico) ==? show ts
 
+instance Arbitrary ClockTime where
+  arbitrary = TOD <$> choose (946681200, 2082754800) <*> choose (0, 99999999999)
+
+-- | Verify our work-around for ghc bug #2519. Taking `diffClockTimes` form
+-- `System.Time`, this test fails with an exception.
+prop_timediffAdd :: ClockTime -> ClockTime -> ClockTime -> Property
+prop_timediffAdd a b c =
+  let fwd = Ganeti.Utils.diffClockTimes a b
+      back = Ganeti.Utils.diffClockTimes b a
+  in addToClockTime fwd (addToClockTime back c) ==? c
+
 -- | Test normal operation for 'chompPrefix'.
 --
 -- Any random prefix of a string must be stripped correctly, including the empty
@@ -350,6 +371,19 @@ prop_splitRecombineEithers es =
         (splitleft, splitright, trail) = splitEithers es
         emptylist = []::[Int]
 
+-- | Tests 'isSubsequenceOf'.
+prop_isSubsequenceOf :: Property
+prop_isSubsequenceOf = do
+  -- isSubsequenceOf only has access to Eq so restricting to a small
+  -- set of numbers and short lists still covers everything it can do.
+  let num = choose (0, 5)
+  forAll (choose (0, 4)) $ \n1 ->
+    forAll (choose (0, 4)) $ \n2 ->
+      forAll (vectorOf n1 num) $ \(a :: [Int]) ->
+        forAll (vectorOf n2 num) $ \b ->
+          let subs = S.fromList $ subsequences b
+          in a `isSubsequenceOf` b == a `S.member` subs
+
 testSuite "Utils"
             [ 'prop_commaJoinSplit
             , 'prop_commaSplitJoin
@@ -367,13 +401,16 @@ testSuite "Utils"
             , 'prop_niceSortKey_equiv
             , 'prop_rStripSpace
             , 'prop_trim
-#ifndef NO_REGEX_PCRE
             , 'case_new_uuid
+#ifdef VERSION_regex_pcre
+            , 'case_new_uuid_regex
 #endif
             , 'prop_clockTimeToString
+            , 'prop_timediffAdd
             , 'prop_chompPrefix_normal
             , 'prop_chompPrefix_last
             , 'prop_chompPrefix_empty_string
             , 'prop_chompPrefix_nothing
             , 'prop_splitRecombineEithers
+            , 'prop_isSubsequenceOf
             ]
